/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2008, 2009  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */

/*
 * Enhancement of dojo.xhr so that:
 * - converts url to the RCM proxy URL format
 * - treats handleAs=json as default, and don't use "json-comment-optional" which has security issues
 * - checks login status and prompts for login dialog if not already authenticated
 * - takes care of session timeout (TODO)
 * 
 * It supports additional arguments compared to dojo.xhr
 *	noLogin : boolean
 * 		specify true if authentication is not required (by default, it prompts for login dialog)
 *	isRawUrl : boolean
 * 		specify true if it doesn't require URL conversion to proxy URL
 *	onBegin : Function
 * 		If authenticated, this function just gets called before making this XHR request
 * 		If not authenticated, this function get called after successful login and right before making this XHR request
 * 		Can be used for start showing visual feedback of XHR processing.
 *	onCancel : Function
 * 		callback function that gets executed when Cancel or close(x) button is pressed in the login dialog
 *	processDeferred : Function
 * 		function(dfd){}	dfd is type of Deferred object
 * 		is used to process Deferred object since return value of cq.xhr is null if not authenticated
 * 	customChecker : Function
 *  handleError : boolean
 *  	set to true in order to handle error in cq.xhr. Default is false
 */
dojo.provide("com.ibm.rational.cm.web._base.xhr");

dojo.require("dojo.i18n");
dojo.require("jazz.core.loader");
dojo.require("com.ibm.rational.cm.web.ui.page.CqLoginController");
dojo.require("com.ibm.rational.cm.web.ui.view.common.MessageDialog");

dojo.requireLocalization("com.ibm.rational.cm.web.ui", "common");

(function(){

var load_async = jazz.core.loader.load_async;
var load_sync = jazz.core.loader.load_sync;
var loginController = com.ibm.rational.cm.cqLoginController;

var cq = com.ibm.rational.cm.web;
var cqNS = "com.ibm.rational.cm.web.ui";
var PROXY_BASE_URL = net.jazz.ajax._contextRoot + "/cqweb/";
var _msgDlg;

dojo.mixin(cq, {
	
	/*
	 * Filter XHR arguments
	 */
	filterArgs: function(args){
		var filteredArgs = args;
		
		// convert to proxy URL unless isRawUrl=true
		if(!filteredArgs.isRawUrl){
			filteredArgs.url = cq.convertUrl(filteredArgs.url);
		}

		// don't use "json-comment-optional" as it has security risk (RATLC01065899), and make default "json"
		if(!filteredArgs.handleAs || filteredArgs.handleAs === "json-comment-optional"){
			filteredArgs.handleAs = "json"; 
		}
		
		return filteredArgs;
	},

	/*
	 * Converts request URL to RCM proxy URL
	 */
	convertUrl: function(url){
		return PROXY_BASE_URL + url;
	},
	
	/*
	 * Checks if a user is authenticated or not (through LoginController)
	 * and prompts login dialog if not already authenticated, and execute callback function after authentication
	 */
	_checkLogin: function(xhr, args){
		// filter arguments
		var filteredArgs = cq.filterArgs(args);
		
		var doCallXhr = function(){
			// execute onBegin function before actually making XHR request
			cq.executeFunc(filteredArgs.onBegin);
			
			// actually make XHR request
			var dfd = xhr(filteredArgs);
			
			// process Deferred object (cq.xhr may return null unlike dojo.xhr when login dialog is shown)
			var processDeferred = filteredArgs.processDeferred;
			if(processDeferred && dojo.isFunction(processDeferred)){
				processDeferred(dfd);
			}
			
			return dfd;			
		}
		
		var callXhr = function(){
			// handle session timeout
			if(filteredArgs.noLogin != true && filteredArgs.handleAs == "json"){
				var origLoad = filteredArgs.load;
				filteredArgs.load = function(response, ioArgs){
					// check for error
					if(filteredArgs.handleError && !cq.checkError(response)){
						cq.executeFunc(filteredArgs.onCancel);
						return response;
					}
					
					// check for session timeout
					if(response.status === "LOGIN_FAIL" || response.STATUS === "LOGIN_FAIL"){
						var loginDfd = new dojo.Deferred();
						loginController.setSessionTimout();
						loginController.process({
							callback: function(){
								return doCallXhr();
							},
							onCancel: function(){
								cq.executeFunc(filteredArgs.onCancel);
								if(filteredArgs.__loginDfd){
									filteredArgs.__loginDfd = null;
								}
							}
						});
						filteredArgs.__loginDfd = loginDfd;
						return loginDfd;
					}
					
					// check with custom checker if specified
					var customChecker = filteredArgs.customChecker;
					if(customChecker && !customChecker(response)){
						return response;
					}
					
					// execute load handler if all above check has passed
					if(origLoad && dojo.isFunction(origLoad)){
						origLoad(response, ioArgs);
					}
					if(filteredArgs.__loginDfd){
						filteredArgs.__loginDfd.callback(response);
					}
					return response;
				}
			}
			
			return doCallXhr();
		}
		
		if(filteredArgs.noLogin == true){ // custom argument to specify that it doesn't require login
			return callXhr();
		}else{ // by default, requires login
			return loginController.process({
				callback: callXhr,
				onCancel: filteredArgs.onCancel
			});
		}
	},

	// additional argument 'origXhr' is added in order to avoid infinite loop
	// when overriding dojo.xhrGet in com.ibm.rational.cm.web.ui.data.Store
	xhrGet: function(args, /* Function */ origXhr){
		var xhr = (origXhr) ? origXhr : dojo.xhrGet;
		return cq._checkLogin(xhr, args);
	},

	xhrPost: function(args){
		return cq._checkLogin(dojo.xhrPost, args);
	},

	/* * General purpose XmlHttpRequest method similar to dojo.xhr in dojo 1.1 * method parameter currently supports "GET", "POST" */
	xhr: function(/*String*/ method, /*dojo.__XhrArgs*/ args){
		switch(method){
			case "POST":
				return cq.xhrPost(args);
			default:
				return cq.xhrGet(args);
		}
	},
	
	// utility function
	executeFunc: function(/* Function */ func){
		if(func && dojo.isFunction(func)){
			return func();
		}
		return null;
	},
	
	// wrapper function to minimize dependency to Jazz
	loadModule: function(/*String*/ module, /*Function*/ callback){
		// if Jazz is not available, we can do this
		// dojo["require"][module];
		// callback();
		load_async(module, callback);
	},
	
	loadModuleSync: function(/*String*/moduleName, /*String*/ cssModuleUri){
		// if Jazz is not available, we can do the same as loadModule
		load_sync(moduleName, cssModuleUri);
	},
	
	/*
	 * Extract error string to be displayed in a dialog from response HTML string
	 */
	getErrStrFromResponseTxt: function(/* String */ txt){
		if(txt){
			var start = txt.toLowerCase().indexOf("<title>");
			var end = txt.toLowerCase().indexOf("</title>", start);
			if(start != -1 && end != -1){
				return txt.substring(start + 7, end);
			}
		}
		return null;
	},
	
	/*
	 * Check if response has error or not, and shows error message dialog if error exists
	 * @return boolean
	 */
	checkError: function(/* Object */ response){
		if(response && dojo.isObject(response) &&
			(response.STATUS != "false") && (response.status != false) && (response.status != "false")){
			return true;
		}else{
			var msg1, msg2;
			if(response && dojo.isObject(response)){
				msg1 = response.ERROR_MESSAGE;
				msg2 = response.DetailedMessage;
			}
			if(!msg1 || msg1.length === 0){
				msg1 = response.MESSAGE;
			}
			if(!msg1 || msg1.length === 0){
				if(response.message){
					if(dojo.isString(response.message)){
						msg1 = response.message;
					}else if(dojo.isObject(response.message)){
						msg1 = response.message.MESSAGE;
					}
				}
			}
			if(!msg1 || msg1.length === 0){ // no ERROR_MESSAGE, so use DetailedMessage instead
				msg1 = msg2;
			}
			if(!msg1 || msg1.length === 0){ // no DetailedMessage as well, so use general error message
				msg1 = dojo.i18n.getLocalization(cqNS, "common").errMsg;
			}
			cq.openMessageDialog(msg1, msg2, response.ExceptionStackTrace, "errorIcon");
			return false;
		}
	},
	
	embedLink: function(str){

		var _findURLEnd = function(text) {
			var len = text.length;
			for (var i=0; i<len; i++){
				var chCode = text.charCodeAt(i);
				if(chCode == 171 || chCode == 9 || chCode == 10 || chCode == 13 || chCode == 32 || chCode == 34){
					return i;
				}
				if (i > 2048){
					// limit the text size
					return -1;
				}
			}
			return len;
		};
	
		var _embedLink = function(protocol, text, isUNC){
			var plen = protocol.length;
			var start = text.indexOf(protocol);
			while(start >= 0){
				var url = text.substring(start + plen);
				var urlEnd	= _findURLEnd(url);
				if(urlEnd < 0){
					break;
				}
				try {
					url = protocol + url.substring(0, urlEnd);

					// RATLC01280621
					// don't encodeURI for UNC links since it can't open when path includes non-ASCII characters
					var link = url.replace(/\\/g,'/');
					link = (isUNC) ? ("file:///" + link) : encodeURI(decodeURI(link));
					link = "<a href='" + link + "' target='_blank'>" + url + "</a>";
					
					text = text.substring(0, start) + link + text.substring(start + url.length);
					start = text.indexOf(protocol, start + link.length);
				}catch(e){
					// ignore invalid url
					start = text.indexOf(protocol, start+1); 
				}
			}
			return text;
		};

		str = _embedLink("http://", str); 
		str = _embedLink("ftp://", str); 
		str = _embedLink("https://", str); 
		str = _embedLink("file://", str); 
		str = _embedLink("\\\\", str, true);
		return str;
	},
	
	openMessageDialog: function(message1, message2, description, iconClass){
		if(!_msgDlg){
			_msgDlg = new com.ibm.rational.cm.web.ui.view.common.MessageDialog({
				title: dojo.i18n.getLocalization(cqNS, "common").lblError
			});
			_msgDlg.resize({ w:500 });
		}
		_msgDlg.setIcon(iconClass);
		var value = message1;
		if(value){
			value = cq.sanitizeString(value);
			value = cq.embedLink(value);
			value = value.replace(/\n/gm, "<br>");
		}		
		_msgDlg.setMessage(value || "");
		var desc = message2 ? message2 + "<br><br>" : "";
		if(description){
			if(dojo.isArray(description)){
				description = description.join("<br>");
			}
			desc = desc + description;
		}
		_msgDlg.setDescription(desc);
		_msgDlg.show();
		return _msgDlg;
	}
});

})();